const React = require('react');
const ReactDOM = require('react-dom');
const joinClasses = require('classnames');
const ExcelColumn = require('./PropTypeShapes/ExcelColumn');
const ResizeHandle = require('./ResizeHandle');
require('./themes/react-data-grid-header.css');

import PropTypes from 'prop-types';

function simpleCellRenderer(objArgs: { column: { name: string } }): ReactElement {
    let headerText = objArgs.column.rowType === 'header' ? objArgs.column.name : '';
    return <div className="widget-HeaderCell__value">{headerText}</div>;
}

class HeaderCell extends React.Component {
    static propTypes = {
        renderer: PropTypes.oneOfType([PropTypes.func, PropTypes.element]).isRequired,
        column: PropTypes.shape(ExcelColumn).isRequired,
        onResize: PropTypes.func.isRequired,
        height: PropTypes.number.isRequired,
        onResizeEnd: PropTypes.func.isRequired,
        className: PropTypes.string,
        //show is filter set
        isFilterSet: PropTypes.bool.isRequired,
        filterValue: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),
        //can filter on this column
        canFilter: PropTypes.bool.isRequired,
        //call show/hide filter window
        showFilterCallback: PropTypes.func.isRequired
    };

    static defaultProps = {
        renderer: simpleCellRenderer
    };

    state: { resizing: boolean } = {resizing: false};

    onDragStart = (e: SyntheticMouseEvent) => {
        this.setState({resizing: true});
        // need to set dummy data for FF
        if (e && e.dataTransfer && e.dataTransfer.setData) e.dataTransfer.setData('text/plain', 'dummy');
    };

    onDrag = (e: SyntheticMouseEvent) => {
        let resize = this.props.onResize || null; // for flows sake, doesnt recognise a null check direct
        if (resize) {
            let width = this.getWidthFromMouseEvent(e);
            if (width > 0) {
                resize(this.props.column, width);
            }
        }
    };

    showHideFilter = () => {
        if (this.props.showFilterCallback != null) {
            this.props.showFilterCallback(this.props.column);
        }
        else {
            console.info('no callback on filter show for column', this.props.column);
        }
    }

    onDragEnd = (e: SyntheticMouseEvent) => {
        let width = this.getWidthFromMouseEvent(e);
        if (width > 95) {
            this.props.onResizeEnd(this.props.column, width);
            if (this._mounted === true) {
                this.setState({resizing: false});
            }
        }
        else {
            this.props.onResizeEnd(this.props.column, 95);
            if (this._mounted === true) {
                this.setState({resizing: false});
            }
        }
    };

    getWidthFromMouseEvent = (e: SyntheticMouseEvent): number => {
        let right = e.pageX || (e.touches && e.touches[0] && e.touches[0].pageX) || (e.changedTouches && e.changedTouches[e.changedTouches.length - 1].pageX);
        let left = ReactDOM.findDOMNode(this).getBoundingClientRect().left;
        return right - left;
    };

    getCell = (): ReactComponent => {
        if (React.isValidElement(this.props.renderer)) {
            // if it is a string, it's an HTML element, and column is not a valid property, so only pass height
            if (typeof this.props.renderer.type === 'string') {
                return React.cloneElement(this.props.renderer, {height: this.props.height});
            }
            return React.cloneElement(this.props.renderer, {column: this.props.column, height: this.props.height});
        }
        return this.props.renderer({column: this.props.column});
    };

    getStyle = (): { width: number; left: number; display: string; position: string; overflow: string; height: number; margin: number; textOverflow: string; whiteSpace: string } => {
        return {
            width: this.props.column.width,
            left: this.props.column.left,
            display: 'inline-block',
            position: 'absolute',
            height: this.props.height - 30,
            marginTop: '30px',
            textOverflow: 'ellipsis',
            whiteSpace: 'nowrap',
        };
    };

    setScrollLeft = (scrollLeft: number) => {
        let node = ReactDOM.findDOMNode(this);
        node.style.webkitTransform = `translate3d(${scrollLeft}px, 0px, 0px)`;
        node.style.transform = `translate3d(${scrollLeft}px, 0px, 0px)`;
    };

    removeScroll = () => {
        let node = ReactDOM.findDOMNode(this);
        if (node) {
            let transform = 'none';
            node.style.webkitTransform = transform;
            node.style.transform = transform;
        }
    };

    dropFilter = () => {
        this.props.column.dropFilter(this.props.column);
    }

    componentDidMount() {
        this._mounted = true;
    }

    componentWillUnmount() {
        this._mounted = false;
    }

    render(): ?ReactElement {
        let resizeHandle;
        if (this.props.column.resizable) {
            resizeHandle = (<ResizeHandle
                onDrag={this.onDrag}
                onDragStart={this.onDragStart}
                onDragEnd={this.onDragEnd}
            />);
        }

        let className = joinClasses({
            'react-grid-HeaderCell': true,
            'react-grid-HeaderCell--resizing': this.state.resizing,
            'react-grid-HeaderCell--locked': this.props.column.locked,
            'filter-set': this.props.isFilterSet === true
        });
        className = joinClasses(className, this.props.className, this.props.column.cellClass);
        //this.props.column.sortable = this.props.canFilter;

        let cell = this.getCell();
        let fil = this.props.height - 30;
        let getFilterValueFunc = (val)=>{
            if(this.props.column.filterHeaderFormatter==null){
                return val.filterTerm;
            }
            return this.props.column.filterHeaderFormatter(val);
        }
        let fVal;
        if(this.props.isFilterSet){
            if(this.props.column.filterCannotBeRemoved===true){
                fVal = (<div className="react-grid-Header__filtred" style={{marginTop: '-20px'}}>getFilterValueFunc(this.props.filterValue);</div>);
            }
            else{
                fVal = (<div className="react-grid-Header__filtred" style={{marginTop: '-20px'}}>
                    {getFilterValueFunc(this.props.filterValue)}
                    {(this.props.column.filterCannotBeRemoved==null||this.props.column.filterCannotBeRemoved==false) && (<button className="react-grid-Header__filtred-cancel" onClick={this.dropFilter}>&times;</button>)}
                </div>);
            }
        }
        return (
            <div className={className} style={this.getStyle()}>
                <div style={{width: this.props.column.width, height: this.props.height - 30}}
                     className="header-cell-wrapper">
                    {cell}
                    {(this.props.column.filterable && (this.props.canFilter || this.props.column.filterFirst)) &&
                    (<button type="button" onClick={this.showHideFilter}
                             className={this.props.column.filterClass ? this.props.column.filterClass : 'react-grid-Header__filter-btn'}>
                        <svg width="16" height="16" viewBox="0 0 16 16" fill="none" xmlns="http://www.w3.org/2000/svg">
                            <path opacity="0.3" d="M6.27001 15.51C6.04001 15.51 5.82001 15.46 5.60001 15.35C5.09001 15.1 4.78001 14.59 4.78001 14.02V6.89L0.180014 1.94C-0.0599864 1.55 -0.0699864 1.05 0.150014 0.65C0.370014 0.25 0.810014 0 1.27001 0H14.3C14.76 0 15.18 0.25 15.41 0.65C15.63 1.05 15.63 1.55 15.39 1.94L15.26 2.1L10.78 6.9V11.76C10.78 12.22 10.56 12.67 10.19 12.95L7.16001 15.22C6.90001 15.41 6.59001 15.51 6.27001 15.51ZM2.95001 2L6.78001 6.11V13L8.78001 11.5V6.11L12.62 2H2.95001Z" fill="#434244"/>
                        </svg>
                    </button>)}
                </div>
                {this.props.isFilterSet &&
                (<div className="react-grid-Header__filtred" style={{marginTop: '-23px'}}>
                    {getFilterValueFunc(this.props.filterValue)}
                    {(this.props.column.filterCannotBeRemoved==null||this.props.column.filterCannotBeRemoved==false) && (<button className="react-grid-Header__filtred-cancel" onClick={this.dropFilter}>&times;</button>)}
                </div>)}
                {resizeHandle}
            </div>
        );
    }
}

module.exports = HeaderCell;
